;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "uniline" "20260825.819"
  "Add▶ ■─UNICODE based diagrams─■ to▶ ■─text files─■."
  '((emacs "29.1")
    (hydra "0.15.0"))
  :url "https://github.com/tbanel/uniline"
  :commit "7760962c0c2fd81fa9b001c343dd1a540dc68329"
  :revdesc "7760962c0c2f"
  :keywords '("convenience" "text"))
